unit DBClass;

interface
uses classes;

type
TCostsData=array[1..20,1..6] of integer;
TDBField=record
  data:TCostsData;
  fieldname:string[255];
end;
TDBFile=file of TDbField;
TDBClass=class
public
  readonly:boolean;
  constructor Create;
  destructor Destroy;
  function FindField(name:string;level:integer):boolean;
  function DeleteField(name:string):boolean;
  procedure AddField(name:string);
  procedure RenameField(newname:string);
  procedure GotoLevel(level:integer);
  function GetFieldCount:integer;
  function GetCurrentFieldName:string;
  function LoadFromFile(path:string):boolean;
  procedure SaveToFile(path:string);
  function GetFieldList:TStrings;
private
  curfield,curlevel:integer;
  curfieldname:string;
  fieldcount:integer;
  dbdata:array of TDBField;
  costsval:array[1..6] of integer;
  procedure SetCosts(index:integer;costs:integer);
  function GetCosts(index:integer):integer;
public
property costs[index:integer]:integer read GetCosts write SetCosts ;default;
end;

implementation
uses SysUtils;

function TDbClass.GetCurrentFieldName;
begin
  result:=curfieldname;
end;

procedure TDBClass.RenameField(newname: string);
begin
  dbdata[curfield].fieldname:=newname;
end;

constructor TDbClass.Create;
var
i:integer;
begin
  curfield:=0;
  curlevel:=0;
  fieldcount:=0;
  SetLength(dbdata,0);
  for I := 1 to 6 do
    costsval[i]:=0;
  curfieldname:='';
  readonly:=false;
end;

function TDbClass.GetFieldList:TStrings;
var
i:integer;
begin
result:=TStringList.Create;
for i := 0 to Length(dbdata) - 1 do
  result.Add(dbdata[i].fieldname);
end;

destructor TDbClass.Destroy;
begin
  dbdata:=nil;
end;

procedure TDbClass.SetCosts(index: Integer; costs: Integer);
begin
if not readonly then
try
if (index>6)or(index<1) then
  raise Exception.Create('[SetCosts]Index not exists(Must be 1..6)') else
  begin
  costsval[index]:=costs;
  if curfield<=Length(dbdata) then
  begin
    if (curlevel<=20)and(curlevel>=1) then
    dbdata[curfield].data[curlevel,index]:=costs else
    raise Exception.Create('[SetCosts]Costs level not exists');
  end else raise Exception.Create('[SetCosts]Curfield out of range of dbdata');
  end;
except
  raise Exception.Create('Unknown Error at SetCosts procedure');
end;
end;

function TDbClass.GetCosts(index: Integer):integer;
begin
try
if (index>6)or(index<1) then
  raise Exception.Create('[SetCosts]Index not exists(Must be 1..6)') else
  result:=costsval[index];
except
  raise Exception.Create('Unknown error at SetCosts procedure');
end;
end;


function TDbClass.FindField(name: string;level:INTEGER):boolean;
var
i:integer;
  j: Integer;
begin
result:=false;
if curfieldname<>name then
begin
for i := 0 to Length(dbdata) - 1 do
  try
  if dbdata[i].fieldname=name then
  begin
  result:=true;
  curfield:=i;
  curlevel:=level;
  curfieldname:=name;
    for j := 1 to 6 do
    costsval[j]:=dbdata[i].data[level,j];
  end;
  except
  raise Exception.Create('[FindField]Array error');
  end;
end else
  begin
  result:=true;
  curlevel:=level;
    for j := 1 to 6 do
    costsval[j]:=dbdata[curfield].data[level,j];
  end;
end;

function TDbClass.DeleteField(name: string):boolean;
var
i,j:integer;
begin
if not readonly then
try
result:=false;i:=0;
while not result do
  if dbdata[i].fieldname=name then
  result:=true else inc(i);
if result then
begin
  for j := i to Length(dbdata) - 2 do
    dbdata[j]:=dbdata[j+1];
  SetLength(dbdata,Length(dbdata)-1);
  dec(fieldcount);
end;
except
  raise Exception.Create('[DeleteField]Unknown Error!');
end;
end;

procedure TDbClass.AddField(name: string);
var
data:TCostsData;
i,j:integer;
begin
if not readonly then
for i := 1 to 20 do
    for j := 1 to 6 do
      data[i,j]:=0;
try
  SetLength(dbdata,Length(dbdata)+1);
  inc(fieldcount);
  curlevel:=1;
  curfieldname:=name;
  curfield:=fieldcount-1;
  dbdata[fieldcount-1].fieldname:=name;
  dbdata[fieldcount-1].data:=data;
except
  raise Exception.Create('[AddField]Unknown error!');
end;
end;

procedure TDbClass.GotoLevel(level: Integer);
var
i:integer;
begin
  curlevel:=level;
  for i := 1 to 6 do
  costsval[i]:=dbdata[curfield].data[level][i];
    
end;

function TDbClass.GetFieldCount;
begin
  result:=fieldcount;
end;

function TDbClass.LoadFromFile(path: string):boolean;
var
f:TDbFile;
log:TextFile;
cur:integer;
data:TDbField;
begin
SetCurrentDir(ExtractFilePath(paramstr(0)));
if FileExists(path) then
  begin
  AssignFile(f,path);
  AssignFile(log,GetCurrentDir+'dblog.txt');
  if fileexists(GetCurrentDir+'dblog.txt') then
    Append(log) else Rewrite(log);
  Reset(f);
  SetLength(dbdata,FileSize(f));
  fieldcount:=FileSize(f);
  Writeln(log,'Load started:'+path+' Record#'+inttostr(fieldcount));
  curfield:=0;
  curlevel:=1;
  cur:=0;
  try
     while not eof(f) do
     begin
     read(f,data);
     dbdata[cur]:=data;
     writeln(log,'Field readed:'+data.fieldname);
     inc(cur);
     end;
  CloseFile(f);
  Writeln(log,'File closed','_____________');
  CloseFile(log);
  except
  raise Exception.Create('[DBClass.LoadFromFile]File read error!');
  end;
  end else
    raise Exception.Create('[DBClass]File not exists! Path:'+path);
end;

procedure TDbClass.SaveToFile(path: string);
var
f:TDBFile;
log:textfile;
i:integer;
begin
SetCurrentDir(ExtractFilePath(paramstr(0)));
try
AssignFile(log,'dblog.txt');
if fileexists('dblog.txt') then
  append(log) else rewrite(log);
AssignFIle(f,path);
Rewrite(f);
Writeln(log,'File saved:'+path+' Record#:'+inttostr(Length(dbdata)));
for i := 0 to Length(dbdata) - 1 do
  begin
  write(f,dbdata[i]);
  writeln(log,'Field writed:'+dbdata[i].fieldname);
  end;
CloseFile(f);
writeln(log,'File Closed','_____________-');
CloseFile(log);
except
raise Exception.Create('[DBCLass]Save error!');
end;
end;

end.
