unit gmUI;

interface

type
  TUIOnDraw = procedure(Id:word;X,Y:integer;Width,Height:word);
  TUIOnChar = procedure(Id:word;Char:char);
  TUIOnKeyDown = procedure(Id:word;Key:integer);
  TUIOnKeyUp = TUIOnKeyDown;
  TUIOnMouseMove = procedure(Id:word;X,Y:integer);
  TUIOnMouseKeyDown = procedure(Id:word;X,Y:integer;Left,Right:boolean);
  TUIOnMouseKeyUp = TUIOnMouseKeyDown;
  TUIOnChangeFocus = procedure(Id:word);
  TUIOnChangeCaption  = procedure(Id:word);
  TUIOnPress = procedure(Id:word);


var
  gmUI_OnChangeFocus:TUIOnChangeFocus;
  gmUI_CursorX,gmUI_CursorY:integer;

procedure gmUIOnChar(Char:char);
procedure gmUIOnKeyDown(Key:integer);
procedure gmUIOnKeyUp(Key:integer);
procedure gmUIOnMouseKeyDown(X,Y:integer;Left,Right:boolean);
procedure gmUIOnMouseKeyUp(X,Y:integer;Left,Right:boolean);
procedure gmUIOnMouseMove(X,Y:integer);

procedure gmUIInit;
procedure gmUIDeInit;

procedure gmUIDraw;
procedure gmUISetFocus(Id:word);

procedure gmUIDelete(Id:word);

function gmUINewInputBox(X,Y:integer;Width,Height:word;
  OnDraw:TUIOnDraw;OnChangeCaption:TUIOnChangeCaption):word;
function gmUINewInputBoxEx(X,Y:integer;Width,Height:word;
  OnDraw:TUIOnDraw;OnChar:TUIOnChar):word;
function gmUINewButton(X,Y:integer;Width,Height:word;
  OnDraw:TUIOnDraw;OnPress:TUIOnPress):word;

function gmUIGetText(Id:word):string;
procedure gmUISetText(Id:word;const Text:string);

function gmUIIsFocused(Id:word):boolean;
function gmUIGetFocus:word;

implementation

uses
  Windows;

type
  PUI = ^TUI;

  TUIOnCharEx = procedure(Id:word;Item:PUI;Char:char);

  TUIType = (
    UI_INPUT_MANUAL,
    UI_INPUT_AUTO,
    UI_BUTTON
    );

  TUI = record
    UIType:TUIType;

    X,Y:integer;
    Width,Height:word;

    Caption:string;

    OnDraw:TUIOnDraw;
    OnChar:TUIOnChar;
    OnCharEx:TUIOnCharEx;
    OnKeyDown:TUIOnKeyDown;
    OnKeyUp:TUIOnKeyUp;
    OnMouseKeyDown:TUIOnMouseKeyDown;
    OnMouseKeyUp:TUIOnMouseKeyUp;
    OnMouseMove:TUIOnMouseMove;
    OnChangeCaption:TUIOnChangeCaption;
    OnPress:TUIOnPress;
    end;

var
  gmUI_Count:word;
  gmUI_Items:array of PUI;
  gmUI_Focus:integer = -1;

function gmUIIsFocused(Id:word):boolean;
begin
  if gmUI_Focus = Id then
    result:=true
  else
    result:=false;  
end;

function gmUIGetFocus:word;
begin
  result:=gmUI_Focus;
end;

function gmUIAdd:word;
var
  i:cardinal;
begin
  if Length(gmUI_Items)>gmUI_Count then
    begin
      for i:=0 to Length(gmUI_Items)-1 do
        if not Assigned(gmUI_Items[i]) then
          begin
            inc(gmUI_Count);
            New(gmUI_Items[i]);
            result:=i;
            gmUI_Focus:=result;
            exit;
          end;
    end;
  inc(gmUI_Count);
  SetLength(gmUI_Items,gmUI_Count);
  New(gmUI_Items[gmUI_Count-1]);
  result:=gmUI_Count-1;
  gmUI_Focus:=result;
end;

procedure gmUIDelete(Id:word);
begin
  if Id>=Length(gmUI_Items)then
    exit;
  if Assigned(gmUI_Items[Id])then
    begin
      dec(gmUI_Count);
      gmUI_Items[Id].Caption:='';
      Dispose(gmUI_Items[Id]);
      gmUI_Items[Id]:=nil;
    end;
end;

procedure gmUIInit;
begin
  gmUI_Count:=0;
  SetLength(gmUI_Items,0);
end;

procedure gmUIDeInit;
var
  i:word;
begin
  if Length(gmUI_Items)>0 then
    for i:=0 to Length(gmUI_Items)-1 do
      gmUIDelete(i);
  gmUI_Count:=0;
  SetLength(gmUI_Items,0);
end;

//---
procedure gmUIOnCharEx(Id:word;Item:PUI;Char:char);
begin
  //case Char of
  case Char of
    #8:{BackSpace}
      begin
        if Item.Caption<>'' then
          if Length(Item.Caption) = 1 then
            Item.Caption:=''
          else
            Item.Caption:=Copy(
              Item.Caption,1,Length(Item.Caption)-1);
        if Assigned(Item.OnChangeCaption) then
          Item.OnChangeCaption(Id);
      end;
    #9:{Tab}
      begin
        if Assigned(gmUI_OnChangeFocus) then
          gmUI_OnChangeFocus(Id);
      end;
    #13:{Enter}
      begin

      end;
    #96://  - 
      begin

      end;
    else
      begin
        Item.Caption:=Item.Caption+Char;
        if Assigned(Item.OnChar)then
          Item.OnChar(Id,Char);
        if Assigned(Item.OnChangeCaption) then
          Item.OnChangeCaption(Id);
      end;
    end;
end;

//---

function gmUINewInputBox(X,Y:integer;Width,Height:word;
  OnDraw:TUIOnDraw;OnChangeCaption:TUIOnChangeCaption):word;
var
  tmp:PUI;
begin
  result:=gmUIAdd;

  tmp:=gmUI_Items[result];

  ZeroMemory(tmp,SizeOf(TUI));

  tmp.X:=X;
  tmp.Y:=Y;
  tmp.Width:=Width;
  tmp.Height:=Height;
  tmp.UIType:=UI_INPUT_AUTO;
  tmp.OnDraw:=OnDraw;
  tmp.OnChangeCaption:=OnChangeCaption;
  tmp.OnCharEx:=gmUIOnCharEx;
end;

function gmUINewInputBoxEx(X,Y:integer;Width,Height:word;
  OnDraw:TUIOnDraw;OnChar:TUIOnChar):word;
var
  tmp:PUI;
begin
  result:=gmUIAdd;

  tmp:=gmUI_Items[result];

  ZeroMemory(tmp,SizeOf(TUI));

  tmp.X:=X;
  tmp.Y:=Y;
  tmp.Width:=Width;
  tmp.Height:=Height;
  tmp.UIType:=UI_INPUT_MANUAL;
  tmp.OnDraw:=OnDraw;
  tmp.OnChar:=OnChar;
end;

function gmUINewButton(X,Y:integer;Width,Height:word;
  OnDraw:TUIOnDraw;OnPress:TUIOnPress):word;
var
  tmp:PUI;
begin
  result:=gmUIAdd;

  tmp:=gmUI_Items[result];

  ZeroMemory(tmp,SizeOf(TUI));

  tmp.X:=X;
  tmp.Y:=Y;
  tmp.Width:=Width;
  tmp.Height:=Height;
  tmp.UIType:=UI_BUTTON;
  tmp.OnDraw:=OnDraw;
  tmp.OnPress:=OnPress;
end;

function gmUIGetText(Id:word):string;
begin
  result:='';
  if Id>=Length(gmUI_Items)then
    exit;
  if Assigned(gmUI_Items[Id])then
    result:=gmUI_Items[Id].Caption;
end;

procedure gmUISetText(Id:word;const Text:string);
begin
  if Id>=Length(gmUI_Items)then
    exit;
  if Assigned(gmUI_Items[Id])then
    gmUI_Items[Id].Caption:=Text;
end;

procedure gmUIDraw;
var
  i:word;
begin
  if gmUI_Count = 0 then
    exit;
  for i:=0 to Length(gmUI_Items)-1 do
    if Assigned(gmUI_Items[i]) then
      if Assigned(gmUI_Items[i].OnDraw) then
        gmUI_Items[i].OnDraw(
          i,
          gmUI_Items[i].X,
          gmUI_Items[i].Y,
          gmUI_Items[i].Width,
          gmUI_Items[i].Height);
end;

procedure gmUISetFocus(Id:word);
begin
  if Id>=Length(gmUI_Items)then
    exit;
  if Assigned(gmUI_Items[Id])then
    gmUI_Focus:=Id;
end;

//---
procedure gmUIOnChar(Char:char);
var
  tmp:PUI;
begin
  if gmUI_Focus = -1 then
    exit;
  if gmUI_Focus>=Length(gmUI_Items) then
    exit;
  if Assigned(gmUI_Items[gmUI_Focus]) then
    begin
      tmp:=gmUI_Items[gmUI_Focus];
      if Assigned(tmp.OnCharEx)then
        tmp.OnCharEx(
          gmUI_Focus,
          tmp,
          Char)
      else
        if Assigned(tmp.OnChar)then
          tmp.OnChar(
            gmUI_Focus,
            Char);
    end;
end;

procedure gmUIOnKeyDown(Key:integer);
begin
  if gmUI_Focus = -1 then
    exit;
  if gmUI_Focus>=Length(gmUI_Items) then
    exit;
  if Assigned(gmUI_Items[gmUI_Focus]) then
    begin
      if Assigned(gmUI_Items[gmUI_Focus].OnKeyDown) then
        gmUI_Items[gmUI_Focus].OnKeyDown(gmUI_Focus,Key);

      case gmUI_Items[gmUI_Focus].UIType of
        UI_INPUT_AUTO,
        UI_INPUT_MANUAL:
          begin
            case Key of
              VK_TAB:
                if Assigned(gmUI_OnChangeFocus) then
                  gmUI_OnChangeFocus(gmUI_Focus);
              end;
          end;
        UI_BUTTON:
          begin
            case Key of
              VK_RETURN:
                if Assigned(gmUI_Items[gmUI_Focus].OnPress) then
                  gmUI_Items[gmUI_Focus].OnPress(gmUI_Focus);
              VK_TAB:
                if Assigned(gmUI_OnChangeFocus) then
                  gmUI_OnChangeFocus(gmUI_Focus);
              end;
          end;
        end;
    end;
end;

procedure gmUIOnKeyUp(Key:integer);
begin
  if gmUI_Focus = -1 then
    exit;
  if gmUI_Focus>=Length(gmUI_Items) then
    exit;
  if Assigned(gmUI_Items[gmUI_Focus]) then
    begin
      if Assigned(gmUI_Items[gmUI_Focus].OnKeyUp) then
        gmUI_Items[gmUI_Focus].OnKeyUp(gmUI_Focus,Key);

      {case gmUI_Items[gmUI_Focus].UIType of
        UI_BUTTON:
          if Assigned(gmUI_Items[gmUI_Focus].OnPress) then
            gmUI_Items[gmUI_Focus].OnPress(gmUI_Focus);
        end;}
    end;
end;

procedure gmUIOnMouseMove(X,Y:integer);
var
  i:word;
begin
  gmUI_CursorX:=X;
  gmUI_CursorY:=Y;
end;

procedure gmUIOnMouseKeyDown(X,Y:integer;Left,Right:boolean);
var
  i:word;
begin
  if gmUI_Count = 0 then
    exit;
  for i:=0 to Length(gmUI_Items)-1 do
    if Assigned(gmUI_Items[i]) then
      begin
        if ((X>=gmUI_Items[i].X)and
           (X<=gmUI_Items[i].X+gmUI_Items[i].Width))and
           ((Y>=gmUI_Items[i].Y)and
           (Y<=gmUI_Items[i].Y+gmUI_Items[i].Height))then
          begin
            if i = gmUI_Focus then
              if Assigned(gmUI_Items[i].OnMouseKeyDown) then
                gmUI_Items[i].OnMouseKeyDown(i,
                  X-gmUI_Items[i].X,
                  Y-gmUI_Items[i].Y,
                  Left,Right);

            case gmUI_Items[i].UIType of
              UI_BUTTON:
                if Assigned(gmUI_Items[i].OnPress) then
                  gmUI_Items[i].OnPress(i);
              end;

            gmUI_Focus:=i;
            exit;
          end;
      end;
end;

procedure gmUIOnMouseKeyUp(X,Y:integer;Left,Right:boolean);
var
  i:word;
begin
  if gmUI_Count = 0 then
    exit;
  for i:=0 to Length(gmUI_Items)-1 do
    if Assigned(gmUI_Items[i]) then
      begin
        if ((X>=gmUI_Items[i].X)and
           (X<=gmUI_Items[i].X+gmUI_Items[i].Width))and
           ((Y>=gmUI_Items[i].Y)and
           (Y<=gmUI_Items[i].Y+gmUI_Items[i].Height))then
          begin
            if i = gmUI_Focus then
              if Assigned(gmUI_Items[i].OnMouseKeyUp) then
                gmUI_Items[i].OnMouseKeyUp(i,
                  X-gmUI_Items[i].X,
                  Y-gmUI_Items[i].Y,
                  Left,Right);

            exit;
          end;
      end;
end;

//---

end.
